;;; ui.el --- User interface for task execution
;;
;; Copyright (C) 2010 Jan Moringen
;;
;; Author: Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
;; Keywords: tasks, user interface
;;
;; This Program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This Program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses>.


;;; Commentary:
;;
;; + `task-renderer'
;;   + `string-task-renderer'
;;     + `mode-line-task-renderer'


;;; History:
;;
;; 0.1 - Initial version


;;; Code:
;;


;;; Class `task-renderer'
;;

(defclass task-renderer ()
  ((task :initarg :task
	 :type    task-child
	 :documentation
	 ""))
  ""
  :abstract t)

(defmethod update ((this task-renderer))
  ""
  )


;;; Class `string-task-renderer'
;;

(defclass string-task-renderer (task-renderer)
  ()
  ""
  :abstract t)

(defmethod make-task-string ((this string-task-renderer))
  ""
  (with-slots (task) this
    (object-print task)))


;;; Class `buffer-task-renderer'
;;

(defclass buffer-task-renderer (string-task-renderer)
  ((overlay :initarg :overlay
	    :type    overlay
	    :documentation
	    ""))
  "")

(defmethod initialize-instance ((this buffer-task-renderer) slots)
  ""
  (when (next-method-p)
    (call-next-method))

  (with-slots (overlay) this
    (setq overlay (make-overlay (point) (+ (point) 1)))))

(defmethod update ((this buffer-task-renderer))
  ""
  (with-slots (overlay) this
    (let ((string (propertize
		   (make-task-string this)
		   'help-echo "bla")))
      (with-current-buffer (overlay-buffer overlay)
	(save-excursion
	  (delete-region (overlay-start overlay) (overlay-end overlay))
	  (goto-char (overlay-start overlay))
	  (insert string)
	  (move-overlay overlay (overlay-start overlay) (point)))))))


;;; Class `mode-line-task-renderer'
;;


(defclass mode-line-task-renderer (string-task-renderer)
  ()
  "")

(defmethod update ((this task-renderer))
  ""
  (when (< (random) 1000000)
    (let ((string (propertize
		   (make-task-string this)
		   'help-echo "bla")))
      (setq header-line-format string)
      (force-mode-line-update)
      (force-window-update)
      (sit-for 0.0))))

(provide 'tasks/ui)


;;; Unit Tests
;;

(eval-when-compile
  (when (require 'ert nil t)

    ;;

    (defclass test-string-task-renderer (string-task-renderer)
      ()
      "")

    ;; Actual test cases

    (ert-deftest tasks/ui-test-string-task-renderer-smoke ()
      ""
      (let* ((task1    (task "bla" :code '(+ 1 2 3)))
	     (renderer (test-string-task-renderer
			"bla"
			:task task1)))
	(make-task-string renderer))

      (let* ((task1    (task
			"bla"
			:code '(+ 1 2 3)
			:description "bla"))
	     (renderer (test-string-task-renderer
			"bla"
			:task task1)))
	(make-task-string renderer))
      )

    (ert-deftest tasks/ui-test-buffer-task-renderer-smoke ()
      ""
      (with-temp-buffer
	(let* ((task1    (make-task numbers (:progress t)
				    ""
			   (dolist (i (number-sequence 1 100))
			     (+ 1 2 3)
			     (sit-for 1.0)
			     (progress (/ (float i) 100.0)))))
	       (renderer (buffer-task-renderer
			  "bla"
			  :task task1)))
	  (object-add-hook task1 'progress-hook (lambda (&rest args) (update renderer)))
	  (execute task1)))
      )

    (ert-deftest tasks/ui-test/mode-line-task-renderer-smoke ()
      ""
      (let* ((task1    (progress-task "bla" :code '(+ 1 2 3)))
	     (renderer (mode-line-task-renderer
			"bla"
			:task task1)))
	(execute task1))
      )

    ))

;;; ui.el ends here
